;;; $Id: add1*.scm,v 1.1 1998/09/28 20:38:55 leavens Exp $

;; Grammar:
;;;    <(tree number)> ::= ( {<(atomic-expression number)>}* )
;;;    <(atomic-expression number)> ::= <number> | <(tree number)>

(load-from-lib "atomic-tree.scm")

;; Examples:
;; (add1* (parse-tree '()) ==> ()
;; (add1* (parse-tree '(3 4 2 1)))
;;                  ==> (4 5 3 2)
;; (add1* (parse-tree '((3 4) (2))))
;;                  ==> ((4 5) (3))
;; (add1* (parse-tree '(1 ((2 () 3)) ((4 ())))))
;;                  ==> (2 ((3 () 4)) ((5 ())))

(define add1*
  ; TYPE: (-> ((tree number)) (tree number))
  (lambda (tr)
    (if (tree-null? tr)
	tr
	(tree-add (add1*-atomic-expression (tree-first tr))
		  (add1* (tree-rest tr))))))

;; Examples:
;; (add1*-atomic-expression (parse-atomic-expression 1)) ==> x
;; (add1*-atomic-expression (parse-atomic-expression '(5 7 (9))))
;;  ==> (6 8 (10))
;; (add1*-atomic-expression (parse-atomic-expression '(7 (9))))
;;  ==> (8 (10))


(define add1*-atomic-expression
  ;; TYPE: (-> ((atomic-expression number))
  ;;           (atomic-expression number))
  (lambda (ae)
    (if (atomic-expression-atom? ae)
	(atom->atomic-expression (+ 1 (atomic-expression->atom ae)))
	(tree->atomic-expression
	 (add1* (atomic-expression->tree ae))))))
