;; $Id: ch5-1.scm,v 1.20 1999/04/25 16:23:52 leavens Exp $

;; Code for EOPL section 5.1 using the chararacter string parser.

;;; Grammar:
;;	    <exp> ::= <integer-literal>
;;		    | <varref>
;;		    | <operator> <operands>
;;	<operator> ::= <varref> | (<exp>)
;;	<operands> ::= ( )
;;	            | (<operand> {, <operand>}*)
;;	 <operand> ::= <exp>
;;	  <varref> ::= <var>

;;; Parsing
;; To use "run", you also need to do
;;        (load "run5-1.scm")
;; To use "read-eval-print", you also need to do
;;        (load "rep5-1.scm")
;; Note that you can't use both of these at the same time.

(load-from-lib "undefine-read-eval-print.scm")
(load-from-lib "undefine-run.scm")
(load-quietly-from-lib "ch5-char-parser.scm")

;;; Domains for this interpreter:
;;;
;;; Environment = <var> -> Denoted-Value
;;; Denoted-Value = Number + Procedure
;;; Expressed-Value = Number + Procedure
;;; Procedure = prim-proc

;;; ADTs used

(load-quietly-from-lib "environments.scm")

;;; Anonymous figure : page 142, 143

(define-record prim-proc (prim-op))

;;; The following have to be after all the specific domains are defined.
(load-quietly-from-lib "ch5-1-denoted-value.scm")
(load-quietly-from-lib "ch5-1-expressed-value.scm")

;;; Figure 5.1.1 : page 141

(define eval-exp
  ;; TYPE: (-> (parsed-exp) Expressed-Value)
  (lambda (exp)
    (variant-case exp
      (lit (datum) (number->expressed datum))
      (varref (var) (denoted->expressed (apply-env init-env var)))
      (app (rator rands)
        (let ((proc (expressed->procedure (eval-exp rator)))
              (args (eval-rands rands)))
          (apply-proc proc args)))
      (else (error "Invalid abstract syntax:" exp)))))

(define eval-rands
  ;; TYPE: (-> ((list parsed-exp)) (list Expressed-Value))
  (lambda (rands)
    (map eval-exp rands)))

;;; Anonymous figure : page 152, 153

(define apply-proc
  ;; TYPE: (-> (Procedure (list Expressed-Value)) Expressed-Value)
  (lambda (proc args)
    (variant-case proc
      (prim-proc (prim-op) (apply-prim-op prim-op args))
      (else (error "Invalid procedure:" proc)))))

(define apply-prim-op
  ;; TYPE: (-> (symbol (list Expressed-Value)) Expressed-Value)
  (lambda (prim-op args)
    (case prim-op
      ((+) (number->expressed (+ (expressed->number (car args))
				 (expressed->number (cadr args)))) )
      ((*) (number->expressed (* (expressed->number (car args))
				 (expressed->number (cadr args)))) )
      ((-) (number->expressed (- (expressed->number (car args))
				 (expressed->number (cadr args)))) )
      ((add1) (number->expressed (+ (expressed->number (car args)) 1)) )
      ((sub1) (number->expressed (- (expressed->number (car args)) 1)) )
      (else (error "Invalid prim-op name:" prim-op)))))

(define prim-op-names  ; TYPE: (list symbol)
  '(+ - * add1 sub1))

(define init-env       ; TYPE: Environment
  (extend-env
    prim-op-names
    (map procedure->denoted (map make-prim-proc prim-op-names))
    the-empty-env))
