;; $Id: ch5-3.scm,v 1.20 1999/04/25 16:23:52 leavens Exp $

;; Code for EOPL section 5.3 using the chararacter string parser.

;;; Grammar:
;;	    <exp> ::= <integer-literal>
;;		    | <varref>
;;		    | <operator> <operands>
;;                  | if <exp> then <exp> else <exp>
;;	            | let <decls> in <exp>
;;	<operator> ::= <varref> | (<exp>)
;;	<operands> ::= ( )
;;	            | (<operand> {, <operand>}*)
;;	 <operand> ::= <exp>
;;	  <varref> ::= <var>
;;	<decls> ::= <decl> {;<decl>}*
;;	 <decl> ::= <var> = <exp>

;;; Syntactic sugars
;;       none

(load-quietly-from-lib "syntax-expand-0.scm")

;;; Top-level Forms
;;      <form> ::= <exp>

(load-quietly-from-lib "eval-form-0.scm")

;;; Parsing
;; To use "run", you also need to do
;;        (load "run.scm")
;; To use "read-eval-print", you also need to do
;;        (load "rep.scm")
;; Note that you can't use both of these at the same time.

(load-quietly-from-lib "undefine-read-eval-print.scm")
(load-quietly-from-lib "undefine-run.scm")
(load-quietly-from-lib "ch5-char-parser.scm")

;;; Domains for this interpreter:
;;;
;;; Environment = <var> -> Denoted-Value
;;; Denoted-Value = Number + Procedure
;;; Expressed-Value = Number + Procedure
;;; Procedure = prim-proc

;;; ADTs used

(load-quietly-from-lib "environments.scm")
(load-quietly-from-lib "procedure.scm")

;;; Anonymous figure : page 146

(define true-value?
  ;; TYPE: (-> (number) boolean)
  (lambda (x)
    (not (zero? x))))

;;; The following have to be after all the specific domains are defined.
(load-quietly-from-lib "ch5-3-denoted-value.scm")
(load-quietly-from-lib "ch5-3-expressed-value.scm")

;;; Figure 5.3.2 : page 150

(define eval-exp
  ;; TYPE: (-> (parsed-exp Environment) Expressed-Value)
  (lambda (exp env)
    (variant-case exp
      (lit (datum) (number->expressed datum))
      (varref (var) (denoted->expressed (apply-env env var)))
      (app (rator rands)
	   (let ((proc (expressed->procedure (eval-exp rator env)))
		 (args (eval-rands rands env)))
	     (apply-proc proc args)))
      (if (test-exp then-exp else-exp)
	  (if (true-value? (expressed->number (eval-exp test-exp env)))
	      (eval-exp then-exp env)
	      (eval-exp else-exp env)))
      (let (decls body)
        (let ((vars (map decl->var decls))
              (exps (map decl->exp decls)))
          (let ((new-env (extend-env vars
				     (map expressed->denoted
					  (eval-rands exps env))
				     env)))
            (eval-exp body new-env))))
      (else (error "Invalid abstract syntax:" exp)))))

(define eval-rands
  ;; TYPE: (-> ((list parsed-exp) Environment) (list Expressed-Value))
  (lambda (rands env)
    (map (lambda (rand) (eval-exp rand env))
         rands)))

;;; Anonymous figure : page 152, 153

(define apply-proc
  ;; TYPE: (-> (Procedure (list Expressed-Value)) Expressed-Value)
  (lambda (proc args)
    (variant-case proc
      (prim-proc (prim-op) (apply-prim-op prim-op args))
      (else (error "Invalid procedure:" proc)))))

(define apply-prim-op
  ;; TYPE: (-> (symbol (list Expressed-Value)) Expressed-Value)
  (lambda (prim-op args)
    (case prim-op
      ((+) (number->expressed (+ (expressed->number (car args))
				 (expressed->number (cadr args)))) )
      ((*) (number->expressed (* (expressed->number (car args))
				 (expressed->number (cadr args)))) )
      ((-) (number->expressed (- (expressed->number (car args))
				 (expressed->number (cadr args)))) )
      ((add1) (number->expressed (+ (expressed->number (car args)) 1)) )
      ((sub1) (number->expressed (- (expressed->number (car args)) 1)) )
      (else (error "Invalid prim-op name:" prim-op)))))

(define prim-op-names  ; TYPE: (list symbol)
  '(+ - * add1 sub1))

(define init-env       ; TYPE: Environment
  (extend-env
    prim-op-names
    (map procedure->denoted (map make-prim-proc prim-op-names))
    the-empty-env))
