;; $Id: ch6-1-4.scm,v 1.10 1998/11/14 06:18:38 leavens Exp $

;; Code from the EOPL chapter 6 revision, figure 6.1.4
;; (arrays in the direct model, call-by-value).

;;; Grammar:
;;          see the file ch6-base.scm for details

(load-from-lib "ch6-base.scm")

;;; Domains for this interpreter:
;;;
;;; Environment = <var> -> Denoted-Value
;;; Denoted-Value = Cell(Storable-Value) + Array(Storable-Value)
;;; Expressed-Value = Number + Procedure + List(Expressed-Value)
;;;                   + Array(Storable-Value) + Void
;;; Procedure = prim-proc + closure
;;; Storable-Value = Number + Procedure + List(Expressed-Value)
;;; Array(T) = {Cell(T)}*

(load-quietly-from-lib "ch6-1-4-storable-value.scm")
(load-quietly-from-lib "ch6-1-4-denoted-value.scm")
(load-quietly-from-lib "ch6-1-4-expressed-value.scm")


;;; Figure 6.1.4 : page 184 %%%%%% New direct arrays %%%%%%

(define denoted->expressed
  ;; TYPE: (-> (Denoted-Value) Expressed-Value)
  (lambda (den-val)
    (if (array? den-val)
        (array->expressed (denoted->array den-val))
        (storable->expressed (cell-ref den-val)))))

(define expressed->denoted
  ;; TYPE: (-> (Expressed-Value) Denoted-Value)
  (lambda (exp-val)
    (if (array? exp-val)
        (array->denoted (array-copy (expressed->array exp-val)))
        (make-cell (expressed->storable exp-val)))))

(define denoted-value-assign!
  ;; TYPE: (-> (Denoted-Value Expressed-Value) void)
  (lambda (den-val exp-val)
    (cond
      ((and (not (array? den-val)) (not (array? exp-val)))  ; changed
       (cell-set! den-val (expressed->storable exp-val)))
      ((and (array? den-val) (array? exp-val))              ; changed
       (array-whole-set! (denoted->array den-val)
			 (expressed->array exp-val)))
      ((and (not (array? den-val)) (array? exp-val))     ; new here and below
       (error "Can't assign array to non-array: " den-val exp-val))
      (else (error "Can't assign non-array to array: " den-val exp-val)))))

(define do-letarray
  ;; TYPE: (-> (number) Denoted-Value)
  (lambda (n)
    (if (not (<= 0 n))  ; added error check vs. text
	(error "do-letarray: bad index")
	(array->denoted (make-array n)))))

(define eval-array-exp
  ;; TYPE: (-> (parsed-exp Environment) Expressed-Value)
  (lambda (exp env)
    (eval-exp exp env)))

;; addition to figure 6.1.4, replaces change to Array ADT on page 184

(define array-element-assign!
  ;; TYPE: (-> ((Array Storable-Value) number Expressed-Value) void)
  (lambda (array index value)
    (if (array? value)
        (error "Cannot assign array to array element: " value)
        (array-set! array index (expressed->storable value)))))
