;;; $Id: ch7-2.tst,v 1.2 1998/12/07 17:22:26 leavens Exp $

(load-quietly-from-lib "run.scm")

(run-regression-tests-equal?
 `(
    ( (run
"let p = proc (x) x := 5
in let a = 3; 
       b = 4
   in begin
        p(a);
        p(b);
        +(a, b)
      end")
    ==> 7 )
    ( (run
"let stackclass = 
     simpleclass (pushed) (stk, localpushed)
      (initialize = method ()
                      begin
                        &localpushed := 0;
                        &stk := emptylist
                      end;
       empty = method () null(&stk);
       push = method (x)
                begin
                  &&pushed := +(&&pushed, 1);
                  &localpushed := +(&localpushed, 1);
                  &stk := cons(x, &stk)
                end;
       pop = method ()
               if null(&stk)
               then error()
               else begin 
                      &&pushed := -(&&pushed, 1);
                      &localpushed := -(&localpushed, 1);
                      &stk := cdr(&stk)
                    end;
       top = method ()
               if null(&stk)
               then error()
               else car(&stk);
       pushed = method () &&pushed;
       localpushed = method () &localpushed)
      &&pushed := 0
in let stack = simpleinstance stackclass
   in begin
       $push(stack, 7);
       $top(stack)
      end")
      ==> 7 )
    ( (run
"let CellClass =
      simpleclass
        () % no class variables
        (value)  % instance variable named value
 	(initialize = method() &value := 0;
         getVal = method() &value;
	 setVal = method(x) &value := x)
         0  % no class variable initialization 
 in let myCell = simpleinstance CellClass
    in begin $setVal(myCell, 3);
             $getVal(myCell)
       end")
      ==> 3 )
    ( (run
"let CellClass =
      simpleclass
        () % no class variables
        (value)  % instance variable named value
 	(initialize = method() &value := 0;
         getVal = method() &value;
	 setVal = method(x) &value := x)
         0  % no class variable initialization 
 in let CellWithStatsClass =
  simpleclass
    (callsMade) % class variable
    (value)     % instance variable
    (initialize = method() &value := 0;
     getVal = method()
       begin
         &&callsMade := add1(&&callsMade);
         &value
       end;
     setVal = method(x)
       begin
         &&callsMade := add1(&&callsMade);
         &value := x
       end;
     numCalls = method()
         &&callsMade
    )
    &&callsMade := 0
  in let o = simpleinstance CellWithStatsClass
     in begin $setVal(o, 3);
              let v = $getVal(o)
              in list(v, $numCalls(o))
        end")
      ==> (3 2) )
    ( (run
"let stackclass = 
     class baseobject, (pushed) (stk, localpushed)
      (initialize = method ()
                      begin
                        &localpushed := 0;
                        &stk := emptylist
                      end;
       empty = method () null(&stk);
       push = method (x)
                begin
                  &&pushed := +(&&pushed, 1);
                  &localpushed := +(&localpushed, 1);
                  &stk := cons(x, &stk)
                end;
       pop = method ()
               if null(&stk)
               then error()
               else begin 
                      &&pushed := -(&&pushed, 1);
                      &localpushed := -(&localpushed, 1);
                      &stk := cdr(&stk)
                    end;
       top = method ()
               if null(&stk)
               then error()
               else car(&stk);
       pushed = method () &&pushed;
       localpushed = method () &localpushed)
      &&pushed := 0
in let boundedstackclass = 
 class stackclass, () (bound)
  (initialize = method () 
                  begin 
                    &bound := 10; 
                    $initialize(super)
                  end;
   push = method (x) 
            if less(&localpushed, &bound)
            then $push(super, x)
            else error();
   setbound = method (x) &bound := x;
   getbound = method () &bound)
  &&pushed := 0
in let stack = simpleinstance boundedstackclass
   in begin
       $setbound(stack, 13);
       $push(stack, 7);
       list($top(stack), $getbound(stack))
      end")
      ==> (7 13) )
    ( (run
"let aclass = 
 class baseobject, (cv) (iv)
  (initialize = method () &iv := 8;
   getcv = method () &&cv;
   getiv = method () &iv)
  &&cv := 6
in let bclass = 
 class aclass, (cv) (iv)
  (initialize = method () 
                  begin 
                    $initialize(super);
                    &iv := 9
                  end)
  &&cv := 7
in let x = simpleinstance bclass
   in let c = $getcv(x);
          i = $getiv(x)
      in list(c,i)")
      ==> (6 8) )
    ( (run
"let Cclass =
  class baseobject,
        () ()
        (initialize = method() noop;
         m1 = method () $m2(self);
         m2 = method () 3
        )
        0
in let Dclass =
  class Cclass,
        () ()
        (m2 = method () 4
        )
        0
in let aD = simpleinstance Dclass
   in $m1(aD)")
      ==> 4 )
    ( (run
"let Cclass =
  class baseobject,
        () ()
        (initialize = method() noop;
         m1 = method () $m2(self);
         m2 = method () 3
        )
        0
in let Dclass =
  class Cclass,
        () ()
        (m2 = method () 4;
         m3 = method () $m2(super)
        )
        0
in let aD = simpleinstance Dclass
   in $m3(aD)")
      ==> 3 )
    ( (run
"let C2class =
  class baseobject,
        () ()
        (initialize = method() noop;
         m1 = method () $m2(self);
         m2 = method () 3;
         m3 = method () 14
        )
        0
in let D2class =
  class C2class,
        () ()
        (m2 = method () $m3(super);
         m3 = method () 5
        )
        0
in list($m1(simpleinstance C2class),
        $m1(simpleinstance D2class))")
      ==> (3 14) )

    )
 )
