;; $Id: ch9-2-1.scm,v 1.1 1999/08/11 16:21:33 leavens Exp $

;; Code for EOPL section 9.2 using the chararacter string parser.

(load-from-lib "ch9-base.scm")

;;; Top-level Forms

(load-from-lib "eval-form-0.scm")

;;; Grammar:
;;	    <exp> ::= <integer-literal>
;;		    | <varref>
;;		    | <operator> <operands>
;;	            | proc <varlist> <exp>
;;                  | if <exp> then <exp> else <exp>
;;	<operator> ::= <varref> | (<exp>)
;;	<operands> ::= ( )
;;	            | (<operand> {, <operand>}*)
;;	 <operand> ::= <exp>
;;	  <varref> ::= <var>
;;	<decls> ::= <decl> {;<decl>}*
;;	 <decl> ::= <var> = <exp>
;;	<varlist> ::= ( ) | ( <vars> )
;;	 <vars> ::= <var> {,<var>}*

;;; Evaluation of expressions

;; Figure 9.2.1

(define eval-exp
  ; TYPE: (-> (parsed-exp Environment) Expressed-Value)
  (lambda (exp env)
    (variant-case exp
      (lit (datum) datum)
      (varref (var) (apply-env env var))
      (app (rator rands)
        (let ((proc (eval-exp rator env))
              (args (eval-rands rands env)))
          (apply-proc proc args)))
      (if (test-exp then-exp else-exp)
        (if (true-value? (eval-exp test-exp env))
            (eval-exp then-exp env)
            (eval-exp else-exp env)))
      (proc (formals body) (make-closure formals body env))
      (else (error "Invalid abstract syntax:" exp)))))

(define eval-rands
  ; TYPE: (-> ((list parsed-exp) Environment) (list Expressed-Value))
  (lambda (rands env)
    (if (null? rands)
        '()
        (cons (eval-exp (car rands) env)
              (eval-rands (cdr rands) env)))))

(define apply-proc
  ; TYPE: (-> (Procedure (list Expressed-Value)) Expressed-Value)
  (lambda (proc args)
    (variant-case proc
      (prim-proc (prim-op) (apply-prim-op prim-op args))
      (closure (formals body env)
        (eval-exp body (extend-env formals args env)))
      (else (error "Invalid procedure:" proc)))))
