;; $Id: ch9-2-2.scm,v 1.1 1999/08/11 16:21:33 leavens Exp $

;; Code for EOPL section 9.2 using the chararacter string parser.

(load-from-lib "ch9-base.scm")

;;; Top-level Forms

(load-from-lib "eval-form-cont.scm")

;;; Grammar:
;;	    <exp> ::= <integer-literal>
;;		    | <varref>
;;		    | <operator> <operands>
;;	            | proc <varlist> <exp>
;;                  | if <exp> then <exp> else <exp>
;;	<operator> ::= <varref> | (<exp>)
;;	<operands> ::= ( )
;;	            | (<operand> {, <operand>}*)
;;	 <operand> ::= <exp>
;;	  <varref> ::= <var>
;;	<decls> ::= <decl> {;<decl>}*
;;	 <decl> ::= <var> = <exp>
;;	<varlist> ::= ( ) | ( <vars> )
;;	 <vars> ::= <var> {,<var>}*


;;; Evaluation of expressions

;;; Figure 9.2.2 : page 300

(define eval-exp
  (lambda (exp env k)
    (variant-case exp
      (lit (datum) (k datum))
      (varref (var) (k (apply-env env var)))
      (app (rator rands)
        (eval-exp rator env
          (lambda (proc)
            (eval-rands rands env
              (lambda (all)
                (apply-proc proc all k))))))
      (if (test-exp then-exp else-exp)
        (eval-exp test-exp env
          (lambda (test)
            (if (true-value? test)
                (eval-exp then-exp env k)
                (eval-exp else-exp env k)))))
      (proc (formals body) (k (make-closure formals body env)))
      (else (error "Invalid abstract syntax:" exp)))))

(define eval-rands
  (lambda (rands env k)
    (if (null? rands)
        (k '())
        (eval-exp (car rands) env
          (lambda (first)
            (eval-rands (cdr rands) env
              (lambda (rest)
                (k (cons first rest)))))))))

(define apply-proc
  (lambda (proc args k)
    (variant-case proc
      (prim-proc (prim-op) (k (apply-prim-op prim-op args)))
      (closure (formals body env)
        (eval-exp body (extend-env formals args env) k))
      (else (error "Invalid procedure:" proc)))))

(define final-valcont
  (lambda (final) final))
