;;; $Id: combinator-lambda-1.scm,v 1.1 1999/02/11 06:25:19 leavens Exp $

;;; An ADT for the untyped lambda 1-argument calculus with
;;; the three constants LS, LK, and LI, representing the combinators
;;; S, K, and I.

;;; Grammar:
;;;  <c-lambda-1-exp> ::= <varref>
;;;                  | (lambda (<var>) <c-lambda-1-exp>)
;;;                  | (<c-lambda-1-exp> <c-lambda-1-exp>)
;;;                  | LS | LK | LI
;;; <varref> ::= <var>
;;; <var> ::= <symbol>

(define-record varref (var))
(define-record lambda (formal body))
(define-record app (rator rand))
(define-record LS ())
(define-record LK ())
(define-record LI ())

(define c-lambda-1-exp?
  ;; TYPE: (-> (datum) boolean)
  (lambda (exp)
    (or (varref? exp)
	(LS? exp)
	(LK? exp)
	(LI? exp)
	(and (lambda? exp)
	     (c-lambda-1-exp? (lambda->body exp)))
	(and (app? exp)
	     (c-lambda-1-exp? (app->rator exp))
	     (c-lambda-1-exp? (app->rand exp))))))

(define parse-c-lambda-1-exp
  ;; TYPE: (-> (datum) c-lambda-1-exp)
  (lambda (exp)
    (cond
     ((symbol? exp)
      (case exp
       ((S) (make-LS))
       ((K) (make-LK))
       ((I) (make-LI))
       (else (make-varref exp))))
     ((and (list? exp) (= (length exp) 3) (eq? (car exp) 'lambda))
      (make-lambda (caadr exp)
		   (parse-c-lambda-1-exp (caddr exp))))
     ((and (list? exp) (= (length exp) 2))
      (make-app (parse-c-lambda-1-exp (car exp))
		 (parse-c-lambda-1-exp (cadr exp))))
     (else (error "parse-c-lambda-1-exp: bad syntax in expression:"
		  exp)))))

(define unparse-c-lambda-1-exp
  ;; (-> (c-lambda-1-exp) datum)
  (lambda (exp)
    (variant-case exp
      (LS () 'S)
      (LK () 'K)
      (LI () 'I)
      (varref (var) var)
      (lambda (formal body)
	(list 'lambda (list formal) (unparse-c-lambda-1-exp body)))
      (app (rator rand) (list (unparse-c-lambda-1-exp rator)
			      (unparse-c-lambda-1-exp rand)))
      (else (error "unparse-c-lambda-1-exp: Invalid abstract syntax"
		   exp)))))

