
(define call/cc call-with-current-continuation)

(define *escape/thunk* "any continuation")

(define escaper
  (lambda (proc)
    (lambda args
      (*escape/thunk*
       (lambda ()
	 (apply proc args))))))

(define receiver-4
  (lambda (continuation)
    (set! *escape/thunk* continuation)
    (*escape/thunk* (lambda () (writeln ";escaper is defined")))))

(define writeln
  (lambda args
    (for-each display args)
    (newline)))

((call/cc receiver-4))


(define abort-operational
  (escaper (lambda (x) x)))
;Value: abort-operational

(+ 1 (abort-operational 2))
;Value: 2

(define abort-defined
  (lambda (e)
    (call/cc (lambda (d) e))))
;Value: abort-defined

(+ 1 (abort-defined 2))
;Value: 3

;;; Relation between escaper and abort-operational
(escaper (lambda (x) 'body))
;=
(lambda (x) (abort-operational 'body))

; For example,
((escaper (lambda (x) (+ 1 x))) 3)
;Value: 4
((lambda (x) (abort-operational (+ 1 x))) 3)
;Value: 4

(+ 7 ((escaper (lambda (x) (+ 1 x))) 3))
;Value: 4

(+ 7 ((lambda (x) (abort-operational (+ 1 x))) 3))
;Value: 4



;;;; Felleisen's C-lift rule
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc e)))
;Value: (2 . body)
;; -->
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (call/cc (lambda (k)
	     (e (lambda (x)
		  (abort-defined (k (context x))))))))
;Value: body



(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (call/cc (lambda (k)
	     (e (lambda (x)
		  (abort-operational (k (context x))))))))
;Value: body


(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (call/cc (lambda (k)
	     (e (escaper (lambda (x)
			   (k (context x))))))))
;Value: body


;;; Felleisen's C_T rule
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (call/cc e))
;Value: body
;;; -->
(let ((e (lambda (continuation) 'body)))
  (e (lambda (x) (abort-operational x))))
;Value: body
;;; or with escaper:
(let ((e (lambda (continuation) 'body)))
  (e (escaper (lambda (x) x))))
;Value: body
(let ((e (lambda (continuation) 'body)))
  (e (lambda (x) (abort-defined x))))
;Value: body

(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (call/cc e))
;Value: 27
;;; -->
(let ((e (lambda (continuation) (continuation 27))))
  (e (lambda (x) (abort-operational x))))
;Value: 27
;;; or with escaper:
(let ((e (lambda (continuation) (continuation 27))))
  (e (escaper (lambda (x) x))))
;Value: 27
(let ((e (lambda (continuation) (continuation 27))))
  (e (lambda (x) (abort-defined x))))
;Value: 27




;;; Felleisen's C_{idem} rule
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (call/cc e)))))
;Value: (2 . body)
;;; -->
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (lambda (x) (abort-operational x)))))))
;Value: (2 . body)
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (lambda (x) (abort-defined x)))))))
;Value: (2 . body)
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (escaper (lambda (x) x)))))))
;Value: (2 . body)


(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (call/cc e)))))
;Value: (2 . 27)
;;; -->
(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (lambda (x) (abort-operational x)))))))
;Value: 27
(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (lambda (x) (abort-defined x)))))))
;Value: (2 . 27)
(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (escaper (lambda (x) x)))))))
;Value: 27



;;; Felleisen's C_{top} rule
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc e)))
;Value: (2 . body)
;;; -->
(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (lambda (x) (abort-operational (k x))))))))
;Value: (2 . body)

(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (lambda (x) (abort-defined (k x))))))))
;Value: (2 . body)

(let ((e (lambda (continuation) 'body))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (escaper (lambda (x) (k x))))))))
;Value: (2 . body)

(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc e)))
;Value: (2 . 27)
;;; -->
(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (lambda (x) (abort-operational (k x))))))))
;Value: (2 . 27)
(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (lambda (x) (abort-defined (k x))))))))
;Value: (2 . 27)
(let ((e (lambda (continuation) (continuation 27)))
      (context (lambda (y) (cons 2 y))))
  (context (call/cc (lambda (k) (e (escaper (lambda (x) (k x))))))))
;Value: (2 . 27)


















