;;; $Id: lambda-1+quote.scm,v 1.8 1999/01/22 17:36:04 leavens Exp $
;;; An ADT for the untyped lambda 1-argument calculus with quoted symbols

;;; Grammar:
;;;  <lambda-1+quote-exp> ::= <varref>
;;;                          | (quote <symbol>)
;;;                          | (lambda (<var>) <lambda-1+quote-exp>)
;;;                          | (<lambda-1+quote-exp> <lambda-1+quote-exp>)
;;; <varref> ::= <var>
;;; <var> ::= <symbol>

(define lambda-1+quote-exp?
  ;; TYPE: (-> (datum) boolean)
  (lambda (exp)
    (or (varref? exp)
	(and (quote? exp) 
	     (symbol? (quote->symbol exp)))
	(and (lambda? exp) 
	     (lambda-1+quote-exp? (lambda->body exp)))
	(and (app? exp)
	     (lambda-1+quote-exp? (app->rator exp))
	     (lambda-1+quote-exp? (app->rand exp))))))

(define varref? symbol?)  ; TYPE: (-> (lambda-1+quote-exp) boolean)

(define quote?   ; TYPE: (-> (lambda-1+quote-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (= (length exp) 2)
	 (eq? (car exp) 'quote)
	 (symbol? (cadr exp)))))

(define lambda?  ; TYPE: (-> (lambda-1+quote-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (= (length exp) 3)
	 (eq? (car exp) 'lambda)
	 (let ((formals (cadr exp)))
	   (and (list? formals)
		(= 1 (length formals))
		(symbol? (car formals)))))))

(define app?  ; TYPE: (-> (lambda-1+quote-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (= (length exp) 2)
	 (not (eq? (car exp) 'quote)))))

(define varref->var   ; TYPE: (-> (lambda-1+quote-exp) symbol)
  ;; REQUIRES: the argument is a varref
  (lambda (vr) vr))

(define quote->symbol   ; TYPE: (-> (lambda-1+quote-exp) symbol)
  ;; REQUIRES: the argument is a quoted expression
  cadr)

(define lambda->formal   ; TYPE: (-> (lambda-1+quote-exp) symbol)
  ;; REQUIRES: the argument is a lambda
  caadr)

(define lambda->body   ; TYPE: (-> (lambda-1+quote-exp) lambda-1+quote-exp)
  ;; REQUIRES: the argument is a lambda
  caddr)

(define app->rator   ; TYPE: (-> (lambda-1+quote-exp) lambda-1+quote-exp)
  ;; REQUIRES: the argument is an application
  car)

(define app->rand    ; TYPE: (-> (lambda-1+quote-exp) lambda-1+quote-exp)
  ;; REQUIRES: the argument is an application expression
  cadr)

(define make-varref
  ;; TYPE: (-> (symbol) lambda-1+quote-exp)
  (lambda (sym)
    sym))

(define make-quote
  ;; TYPE: (-> (symbol) lambda-1+quote-exp)
  (lambda (sym)
    (list 'quote sym)))

(define make-lambda
  ;; TYPE: (-> (symbol lambda-1+quote-exp) lambda-1+quote-exp)
  (lambda (var body)
    (list 'lambda (list var) body)))

(define make-app
  ;; TYPE: (-> (lambda-1+quote-exp lambda-1+quote-exp) lambda-1+quote-exp)
  list)
  
(define parse-lambda-1+quote
  ;; TYPE: (-> (datum) lambda-1+quote-exp)
  (lambda (exp)
    (cond
     ((symbol? exp) (make-varref exp))
     ((quote? exp) (make-quote (cadr exp)))
     ((lambda? exp)
      (make-lambda (caadr exp)
		   (parse-lambda-1+quote (caddr exp))))
     ((app? exp)
      (make-app (parse-lambda-1+quote (car exp))
		(parse-lambda-1+quote (cadr exp))))
     (else (error "parse-lambda: bad syntax in expression: " exp)))))
