;;; $Id: lambda-1.scm,v 1.8 1999/01/22 17:04:22 leavens Exp $
;;; An ADT for the untyped lambda 1-argument calculus without constants

;;; Grammar:
;;;  <lambda-1-exp> ::= <varref>
;;;                          | (lambda (<var>) <lambda-1-exp>)
;;;                          | (<lambda-1-exp> <lambda-1-exp>)
;;; <varref> ::= <var>
;;; <var> ::= <symbol>

(define lambda-1-exp?
  ;; TYPE: (-> (datum) boolean)
  (lambda (exp)
    (or (varref? exp)
	(and (lambda? exp) 
	     (lambda-1-exp? (lambda->body exp)))
	(and (app? exp)
	     (lambda-1-exp? (app->rator exp))
	     (lambda-1-exp? (app->rand exp))))))

(define varref? symbol?)  ; TYPE: (-> (lambda-1-exp) boolean)

(define lambda?  ; TYPE: (-> (lambda-1-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (= (length exp) 3)
	 (eq? (car exp) 'lambda)
	 (let ((formals (cadr exp)))
	   (and (list? formals)
		(= 1 (length formals))
		(symbol? (car formals)))))))

(define app?  ; TYPE: (-> (lambda-1-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (= (length exp) 2))))

(define varref->var   ; TYPE: (-> (lambda-1-exp) symbol)
  ;; REQUIRES: the argument is a varref
  (lambda (vr) vr))

(define lambda->formal   ; TYPE: (-> (lambda-1-exp) symbol)
  ;; REQUIRES: the argument is a lambda
  caadr)

(define lambda->body   ; TYPE: (-> (lambda-1-exp) lambda-1-exp)
  ;; REQUIRES: the argument is a lambda
  caddr)

(define app->rator   ; TYPE: (-> (lambda-1-exp) lambda-1-exp)
  ;; REQUIRES: the argument is an application
  car)

(define app->rand    ; TYPE: (-> (lambda-1-exp) lambda-1-exp)
  ;; REQUIRES: the argument is an application expression
  cadr)

(define make-varref
  ;; TYPE: (-> (symbol) lambda-1-exp)
  (lambda (sym)
    sym))

(define make-lambda
  ;; TYPE: (-> (symbol lambda-1-exp) lambda-1-exp)
  (lambda (var body)
    (list 'lambda (list var) body)))

(define make-app
  ;; TYPE: (-> (lambda-1-exp lambda-1-exp) lambda-1-exp)
  (lambda (rator rand)
    (list rator rand)))
  
(define parse-lambda-1
  ;; TYPE: (-> (datum) lambda-1-exp)
  (lambda (exp)
    (cond
     ((symbol? exp) (make-varref exp))
     ((lambda? exp)
      (make-lambda (caadr exp)
		   (parse-lambda-1 (caddr exp))))
     ((app? exp)
      (make-app (parse-lambda-1 (car exp))
		(parse-lambda-1 (cadr exp))))
     (else (error "parse-lambda: bad syntax in lambda expression: " exp)))))
