;;; $Id: lambda-multiple+if.scm,v 1.7 1999/01/22 17:36:04 leavens Exp $
;;; An ADT for the untyped lambda multiple-argument calculus
;;; with quoted symbols and if expressions.

;;;	<lcm+if-exp> ::= <varref>
;;;               | ( quote <symbol> )
;;;		  | ( lambda ( {<var>}* ) <lcm+if-exp> )
;;;		  | ( if <lcm+if-exp> <lcm+if-exp> <lcm+if-exp> )
;;;		  | ( <lcm+if-exp> {<lcm+if-exp>}* )
;;;     <var> ::= <symbol>
;;;     <varref> ::= <symbol>

(load-quietly-from-lib "all.scm")

(define lcm+if-exp?
  ;; TYPE: (-> (datum) boolean)
  (lambda (exp)
    (or (varref? exp)
	(and (quote? exp) 
	     (symbol? (quote->symbol exp)))
	(and (lambda? exp) 
	     (lcm+if-exp? (lambda->body exp)))
	(and (if? exp)
	     (lcm+if-exp? (if->test-exp exp))
	     (lcm+if-exp? (if->then-exp exp))
	     (lcm+if-exp? (if->else-exp exp)))
	(and (app? exp)
	     (lcm+if-exp? (app->rator exp))
	     (all lcm+if-exp? (app->rands exp))))))

(define varref? symbol?)  ; TYPE: (-> (lcm+if-exp) boolean)

(define quote?   ; TYPE: (-> (lcm+if-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (= (length exp) 2)
	 (eq? (car exp) 'quote)
	 (symbol? (cadr exp)))))

(define lambda?  ; TYPE: (-> (lcm+if-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (= (length exp) 3)
	 (eq? (car exp) 'lambda)
	 (let ((formals (cadr exp)))
	   (and (list? formals)
		(all symbol? formals))))))

(define if?  ; TYPE: (-> (lcm+if-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (= (length exp) 4)
	 (eq? (car exp) 'if))))

(define app?  ; TYPE: (-> (lcm+if-exp) boolean)
  (lambda (exp)
    (and (list? exp)
	 (>= (length exp) 1)
	 (let ((operator (car exp)))
	   (and (not (eq? operator 'quote))
		(not (eq? operator 'lambda))
		(not (eq? operator 'if)))))))

(define varref->var   ; TYPE: (-> (lcm+if-exp) symbol)
  ;; REQUIRES: the argument is a varref
  (lambda (vr) vr))

(define quote->symbol   ; TYPE: (-> (lcm+if-exp) symbol)
  ;; REQUIRES: the argument is a quoted expression
  cadr)

(define lambda->formals   ; TYPE: (-> (lcm+if-exp) (list symbol))
  ;; REQUIRES: the argument is a lambda
  (lambda (exp)
    (cadr exp)))

(define lambda->body   ; TYPE: (-> (lcm+if-exp) lcm+if-exp)
  ;; REQUIRES: the argument is a lambda
  (lambda (exp)
    (caddr exp)))

(define app->rator   ; TYPE: (-> (lcm+if-exp) lcm+if-exp)
  ;; REQUIRES: the argument is an application
  (lambda (exp)
    (car exp)))

(define app->rands    ; TYPE: (-> (lcm+if-exp) (list lcm+if-exp))
  ;; REQUIRES: the argument is an application expression
  (lambda (exp)
    (cdr exp)))

(define if->test-exp
  ; TYPE: (-> (lcm+if-exp) lcm+if-exp)
  cadr)

(define if->then-exp
  ; TYPE: (-> (lcm+if-exp) lcm+if-exp)
  caddr)

(define if->else-exp
  ; TYPE: (-> (lcm+if-exp) lcm+if-exp)
  cadddr)

(define make-varref
  ;; TYPE: (-> (symbol) lcm+if-exp)
  (lambda (sym)
    sym))

(define make-quote
  ;; TYPE: (-> (symbol) lcm+if-exp)
  (lambda (sym)
    (list 'quote sym)))

(define make-lambda
  ;; TYPE: (-> ((list symbol) lcm+if-exp) lcm+if-exp)
  (lambda (formals body)
    (list 'lambda formals body)))

(define make-if
  ;; TYPE: (-> (lcm+if-exp lcm+if-exp lcm+if-exp) lcm+if-exp)
  (lambda (test consequent alternative)
    (list 'if test consequent alternative)))

(define make-app
  ;; TYPE: (-> (lcm+if-exp (list lcm+if-exp)) lcm+if-exp)
  cons)
  
(define parse-lcm+if-exp
  ;; TYPE: (-> (datum) lcm+if-exp)
  (lambda (exp)
    (cond
     ((symbol? exp) (make-varref exp))
     ((quote? exp) (make-quote (cadr exp)))
     ((lambda? exp)
      (make-lambda (cadr exp) (parse-lcm+if-exp (caddr exp))))
     ((if? exp) (make-if (parse-lcm+if-exp (cadr exp))
			 (parse-lcm+if-exp (caddr exp))
			 (parse-lcm+if-exp (cadddr exp))))
     ((app? exp) (make-app (parse-lcm+if-exp (car exp))
			   (map parse-lcm+if-exp (cdr exp))))
     (else (error "parse-lcm+if-exp: bad syntax in expression: " exp)))))
