;;; $Id: record-lambda-1-exp.scm,v 1.3 1998/10/09 19:48:37 leavens Exp $
;;; An ADT for the untyped lambda 1-argument calculus with numeric literals

;;; Grammar:
;;;  <lambda-1-exp> ::= <varref>
;;;                   | (lambda (<var>) <lambda-1-exp>)
;;;                   | (<lambda-1-exp> <lambda-1-exp>)
;;; <varref> ::= <var>
;;; <var> ::= <symbol>

(define-record varref (var))
(define-record lambda (formal body))
(define-record app (rator rand))

(define lambda-1-exp?
  ;; TYPE: (-> (datum) boolean)
  (lambda (exp)
    (or (varref? exp)
	(and (lambda? exp)
	     (lambda-1-exp? (lambda->body exp)))
	(and (app? exp)
	     (lambda-1-exp? (app->rator exp))
	     (lambda-1-exp? (app->rand exp))))))

(define parse-lambda-1-exp
  ;; TYPE: (-> (datum) lambda-1-exp)
  (lambda (exp)
    (cond
     ((symbol? exp) (make-varref exp))
     ((and (list? exp) (= (length exp) 3) (eq? (car exp) 'lambda))
      (make-lambda (caadr exp)
		   (parse-lambda-1-exp (caddr exp))))
     ((and (list? exp) (= (length exp) 2))
      (make-app (parse-lambda-1-exp (car exp))
		 (parse-lambda-1-exp (cadr exp))))
     (else (error "parse-lambda-1-exp: bad syntax in expression:"
		  exp)))))

(define unparse-lambda-1-exp
  ;; (-> (lambda-1-exp) datum)
  (lambda (exp)
    (variant-case exp
      (varref (var) var)
      (lambda (formal body)
	(list 'lambda (list formal) (unparse-lambda-1-exp body)))
      (app (rator rand) (list (unparse-lambda-1-exp rator)
			      (unparse-lambda-1-exp rand)))
      (else (error "unparse-lambda-1-exp: Invalid abstract syntax"
		   exp)))))
