;;; $Id: simple-arith-expr.scm,v 1.2 1999/01/10 19:00:47 leavens Exp $

;;; Helping procedures for the following grammar.
;;;   <simple-arith-expr> ::= <number>
;;;                       | (<op> <simple-arith-expr> <simple-arith-expr>)
;;;   <op> ::= + | - | *

(define literal?
  ;; TYPE: (-> (simple-arith-expr) boolean)
  (lambda (x)
    (number? x)))

(define make-literal
  ;; TYPE: (-> (number) simple-arith-expr)
  (lambda (num)
    num))

(define literal->number
  ;; TYPE: (-> (simple-arith-expr) number)
  (lambda (sae)
    (if (literal? sae)
	sae
	(error "literal->number called with non-number" sae))))

(define simple-arith-expr:permitted-ops
  ;; TYPE: (list symbol)
  '(+ - * /))

(define op-call?
  ;; TYPE: (-> (simple-arith-expr) boolean)
  (lambda (x)
    (and (list? x)
	 (= (length x) 3)
	 (memq (car x) simple-arith-expr:permitted-ops))))

(define make-op-call
  ;; TYPE: (-> (symbol simple-arith-expr simple-arith-exp) simple-arith-expr)
  (lambda (op left-arg right-arg)
    ;; REQUIRES: op is a member of simple-arith-expr:permitted-ops
    (list op left-arg right-arg)))

(define op-call->op
  ;; TYPE: (-> (simple-arith-expr) symbol)
  (lambda (sae)
    (if (op-call? sae)
	(car sae)
	(error "op-call->op called with non-op-call" sae))))

(define op-call->left-arg
  ;; TYPE: (-> (simple-arith-expr) simple-arith-expr)
  (lambda (sae)
    (if (op-call? sae)
	(cadr sae)
	(error "op-call->left-arg called with non-op-call" sae))))

(define op-call->right-arg
  ;; TYPE: (-> (simple-arith-expr) simple-arith-expr)
  (lambda (sae)
    (if (op-call? sae)
	(caddr sae)
	(error "op-call->right-arg called with non-op-call" sae))))

(define simple-arith-expr?
  ;; TYPE: (-> (datum) boolean)
  (lambda (x)
    (or (literal? x)
	(and (op-call? x)
	     (simple-arith-expr? (op-call->left-arg x))
	     (simple-arith-expr? (op-call->right-arg x))))))

(define parse-simple-arith-expr
  ;; TYPE: (-> (datum) simple-arith-expr)
  (lambda (exp)
    (cond
     ((number? exp) (make-literal exp))
     ((and (list? exp)
	   (= (length exp) 3)
	   (memq (car exp) simple-arith-expr:permitted-ops))
      (make-op-call (car exp)
		    (parse-simple-arith-expr (cadr exp))
		    (parse-simple-arith-expr (caddr exp))))
     (else (error "parse-simple-arith-expr: bad syntax:" exp)))))
