;;; $Id: subst.scm,v 1.3 1999/01/22 16:39:17 leavens Exp $

;; Grammar:
;; <sym-exp>::= <symbol> | <s-list>
;; <s-list> ::= ( {<sym-exp>}* )

(load-from-lib "sym-exp")

;; Examples:
;; (subst 'y 'x (parse-s-list '())) ==> ()
;; (subst 'y 'x (parse-s-list '(x y () (x)))) ==> (y y () (y))
;; (subst 'y 'x (parse-s-list '(y () (x)))) ==> (y () (y))

(define subst
  ;; TYPE: (-> (symbol symbol (list sym-exp)) (list sym-exp))
  (lambda (new old slist)
    (if (null? slist)
	slist
	(cons (subst-sym-exp new old (car slist))
		    (subst new old (cdr slist))))))

;; Examples:
;; (subst-sym-exp 'x 'a (parse-sym-exp 'a)) ==> x
;; (subst-sym-exp 'x 'a (parse-sym-exp 'b)) ==> b
;; (subst-sym-exp 'x 'a (parse-sym-exp '(x y (x))))
;;  ==> (x y (x))
;; (subst-sym-exp 'x 'a (parse-sym-exp '(y (x))))
;;  ==> (y (x))


(define subst-sym-exp
  ;; TYPE: (-> (symbol symbol sym-exp) sym-exp)
  (lambda (new old symexp)
    (if (sym-exp-symbol? symexp)
	(if (eq? old (sym-exp->symbol symexp))
	    (symbol->sym-exp new)
	    symexp)
	(s-list->sym-exp
	 (subst new old (sym-exp->s-list symexp))))))
