;; $Id: syntax-expand-OOP.scm,v 1.6 1999/08/11 16:28:13 leavens Exp $

;; A version of syntax-expand that handles all the syntax,
;; even from the exercises, in chapter 7 of EOPL,
;; with the following...

;;; Syntactic sugars
;;        let x1 = e1; ...; xn = en in body
;;    ==>
;;        (proc (x1, ..., xn) body) (e1, ..., en)
;;
;;        letrec x1 = e1; ...; xn = en in body
;;    ==>
;;        let x1 = 0; ...; xn = 0
;;        in let g1 = e1; ...; gn = en
;;           in begin
;;                begin x1 := e1; begin ...; begin xn := en; 0 end ... end;
;;                body
;;              end
;;
;;        letrecproc p1(f1s) = e1; ...; pn(fns) = en in body
;;    ==>
;;        letrec p1 = proc(f1s) = e1; ...; pn = proc(fns) = en in body

;;; Grammar:
;;	    <exp> ::= <integer-literal>
;;		    | <varref>
;;		    | <instance-var> | <class-var>
;;                  | <instance-var> := <exp>
;;                  | <class-var> := <exp>
;;                  | method <varlist> <exp>
;;                  | $ <var> (<exps>)
;;                  | simpleinstance <exp>
;;                  | simpleclass <varlist> <varlist> <methdecls> <exp>
;;                  | class <exp>, <varlist> <varlist> <methdecls> <exp>
;;                  | instance <exp>, <exp>, <varlist>, <methdecls>
;;                  | $ <var> <super-rands>
;;		    | <operator> <operands>
;;	            | let <decls> in <exp>
;;                  | if <exp> then <exp> else <exp>
;;                  | begin exp1; exp2 end
;;	            | proc <varlist> <exp>
;;	            | <var> := <exp>
;;                  | letrec <decls> in <exp>
;;                  | letrecproc <procdecls> in <exp>
;;	<operator> ::= <varref> | (<exp>)
;;	<operands> ::= ( )
;;	            | (<operand> {, <operand>}*)
;;	 <operand> ::= <exp>
;;	<super-rands> ::= ( super )
;;	            | (super , {, <exp>}*)
;;	  <varref> ::= <var>
;;         <decls> ::= <decl> {;<decl>}*
;;	    <decl> ::= <var> = <exp>
;;	 <varlist> ::= ( ) | ( <vars> )
;;	    <vars> ::= <var> {,<var>}*
;;     <procdecls> ::= <procdecl> {;<procdecl>}*
;;	<procdecl> ::= <var> <varlist> = <exp>

(define cs342:next-symbol
  (let ((c 0))
    (lambda ()
      (set! c (+ c 1))
      (string->symbol (string-append "g" (number->string c))))))

(define cs342:syntax-expand-decl
  ; TYPE: (-> (parsed-decl) parsed-decl)
  (lambda (decl)
    (make-decl (decl->var decl)
	       (syntax-expand (decl->exp decl)))))

(define syntax-expand
  ; TYPE: (-> (parsed-exp) parsed-exp)
  (lambda (parsed-exp)
    (variant-case parsed-exp
      (lit () parsed-exp)
      (varref () parsed-exp)
      (i-varref () parsed-exp)
      (c-varref () parsed-exp)
      (i-varassign (var exp)
	 (make-i-varassign var (syntax-expand exp)))
      (c-varassign (var exp)
	 (make-c-varassign var (syntax-expand exp)))
      (method (formals body)
	(make-method formals (syntax-expand body)))
      (meth-app (name rands)
        (let ((args (map syntax-expand rands)))
	  (make-meth-app name args)))
      (new-simpleinst (class-exp)
	(make-new-simpleinst (syntax-expand class-exp)))
      (new-simpleclass (c-vars i-vars methdecls init-exp)
	(make-new-simpleclass c-vars i-vars
			      (map cs342:syntax-expand-decl methdecls)
			      (syntax-expand init-exp)))
      (new-class (parent-exp c-vars i-vars methdecls init-exp)
	(make-new-class (syntax-expand parent-exp)
			c-vars i-vars
			(map cs342:syntax-expand-decl methdecls)
			(syntax-expand init-exp)))
      (super-meth-app (name rands)
        (let ((args (map syntax-expand rands)))
	  (make-super-meth-app name args)))
      (new-instance (class-exp parent-exp i-vars methdecls)
	(make-new-instance (syntax-expand class-exp)
			   (syntax-expand parent-exp)
			   i-vars
			   (map cs342:syntax-expand-decl methdecls)))
      (app (rator rands)
        (let ((proc (syntax-expand rator))
	      (args (map syntax-expand rands)))
	  (make-app proc args)))
      (let (decls body)
	(let ((vars (map decl->var decls))
              (exps (map (lambda (decl)
			   (syntax-expand (decl->exp decl)))
			 decls)))
	  (make-app (make-proc vars (syntax-expand body))
		    exps)))
      (if (test-exp then-exp else-exp)
	  (make-if (syntax-expand test-exp)
		   (syntax-expand then-exp)
		   (syntax-expand else-exp)))
      (proc (formals body) 
	(make-proc formals (syntax-expand body)))
      (varassign (var exp)
	(make-varassign var (syntax-expand exp)))
      (begin (exp1 exp2)
	(make-begin (syntax-expand exp1)
		    (syntax-expand exp2)))
      (letrec (decls body)
	(let ((vars (map decl->var decls))
              (exps (map decl->exp decls)))
	  (syntax-expand
	   (make-let
	    (map make-decl vars (map
				 (lambda (ignored) (make-lit 0))
				 exps))
	    (make-begin	       
	     (letrec ((new-vars (map (lambda (ignored)
				       (cs342:next-symbol))
				     vars))
		      (make-begins
		       ; TYPE: (-> ((list <var>) (list <exp>)) <exp>)
		       (lambda (vlist elist)
			 (if (null? vlist)
			     (make-lit 0)
			     (make-begin
			      (make-varassign (car vlist)
					      (car elist))
			      (make-begins (cdr vlist) (cdr elist)))))))
	       (make-let (map make-decl new-vars exps)
			 (make-begins vars
				      (map make-varref new-vars))))
	     body)))))
      (letrecproc (procdecls body)
	(let ((vars (map procdecl->var procdecls))
	      (formals-list (map procdecl->formals procdecls))
              (bodies (map procdecl->body procdecls)))
	  (syntax-expand
	   (make-letrec
	    (map make-decl vars (map make-proc formals-list bodies))
	    body))))
      (else
       (displayln "Warning: syntax-expand not expanding : " parsed-exp)
       parsed-exp))))
