;; $Id: syntax-expand-let.scm,v 1.5 1997/12/19 03:34:53 leavens Exp $

;; syntax-expand that expands let expressions,
;; and also handles the synatx of some exercises:
;;      letmutable, letdynamic, dynassign

;;; Grammar:
;;	    <exp> ::= <integer-literal>
;;		    | <varref>
;;		    | <operator> <operands>
;;	            | let <decls> in <exp>
;;                  | if <exp> then <exp> else <exp>
;;                  | begin exp1; exp2 end
;;                  | letmutable <decls> in <exp>
;;                  | letdynamic <decls> in <exp>
;;                  | dynassign <var> := <exp> during <exp>
;;	            | proc <varlist> <exp>
;;	            | <var> := <exp>
;;	<operator> ::= <varref> | (<exp>)
;;	<operands> ::= ( )
;;	            | (<operand> {, <operand>}*)
;;	 <operand> ::= <exp>
;;	  <varref> ::= <var>
;;	<decls> ::= <decl> {;<decl>}*
;;	 <decl> ::= <var> = <exp>
;;	<varlist> ::= ( ) | ( <vars> )
;;	 <vars> ::= <var> {,<var>}*

;;; Syntactic sugars
;;        let x1 = e1; ...; xn = en in body
;;    ==>
;;        (proc (x1, ..., xn) body) (e1, ..., en)

(define syntax-expand
  ; TYPE: (-> (parsed-exp) parsed-exp)
  (lambda (parsed-exp)
    (variant-case parsed-exp
      (lit () parsed-exp)
      (varref () parsed-exp)
      (app (rator rands)
        (let ((proc (syntax-expand rator))
	      (args (map syntax-expand rands)))
	  (make-app proc args)))
      (let (decls body)
	(let ((vars (map decl->var decls))
              (exps (map (lambda (decl)
			   (syntax-expand (decl->exp decl)))
			 decls)))
	  (make-app (make-proc vars (syntax-expand body))
		    exps)))
      (if (test-exp then-exp else-exp)
	  (make-if (syntax-expand test-exp)
		   (syntax-expand then-exp)
		   (syntax-expand else-exp)))
      (proc (formals body) 
	(make-proc formals (syntax-expand body)))
      (varassign (var exp)
	(make-varassign var (syntax-expand exp)))
      (begin (exp1 exp2)
	(make-begin (syntax-expand exp1)
		    (syntax-expand exp2)))
      (letmutable (decls body)
	 (make-letmutable (map cs342:syntax-expand-decl decls)
			  (syntax-expand body)))
      (letdynamic (decls body)
	  (make-letdynamic (map cs342:syntax-expand-decl decls)
			   (syntax-expand body)))
      (dynassign (var exp body)
	 (make-dynassign var (syntax-expand exp) (syntax-expand body)))
      (else
       (displayln "Warning: syntax-expand not expanding : " parsed-exp)
       parsed-exp))))

(define cs342:syntax-expand-decl
  ; TYPE: (-> (parsed-decl) parsed-decl)
  (lambda (decl)
    (make-decl (decl->var decl)
	       (syntax-expand (decl->exp decl)))))
