;;; $Id: type-check-down-type.scm,v 1.3 1999/06/23 16:57:22 leavens Exp $

(define tc:down-type
  ;; TYPE: (-> (type-expr (pair type-expr type-expr)) type-expr)
  (lambda (declared-type abst-rep-pair)
    ;; ENSURES: result is a type-expr like declared-type but in which
    ;;          each sub-expression that unifies with the car of abst-rep-pair
    ;;          has been substituted by the cdr of abst-rep-pair under the
    ;;          unifying substitution.
;    (displayln "tc:down-type: abst-rep-pair = " abst-rep-pair)
    (let* ((result (tc:deep-down-type (tc:type-expr-uniquify declared-type)
				   abst-rep-pair
				   (tc:env-empty)))
	   (expr (car result))
	   (env (cdr result)))
      ;; (displayln "expr = " expr)
      ;; (displayln "env = " env)
      (tc:subst-bindings env expr))))

(define tc:deep-down-type
  ;; TYPE: (-> (type-expr (pair type-expr type-expr)
  ;;            (env uniq-sym type-expr))
  ;;           (pair type-expr (env uniq-sym type-expr)))
  (lambda (up-type abst-rep-pair env)
    (let* ((uniq-rep (tc:type-expr-uniquify abst-rep-pair))
	   (abst-type (car uniq-rep))
	   (rep-type (cdr uniq-rep)))
      ;; (displayln "abst-type: " abst-type ", rep-type: " rep-type)
      (letrec ((down-type-list
		;; TYPE: (-> ((list type-expr))
		;;           (pair (list type-expr)
		;;                 (env uniq-sym type-expr)))
		(lambda (up-types new-env)
		  ;; Each type-expr in the list is unified in an
		  ;; environment formed by the unification of the preceding
		  ;; type-exprs in the list.
		  (if (null? up-types)
		      (cons '() new-env)
		      (let* ((res-first (tc:deep-down-type (car up-types)
							uniq-rep
							new-env))
			     (res-rest (down-type-list (cdr up-types)
						       (cdr res-first))))
			(cons (cons (car res-first) (car res-rest))
			      (cdr res-rest))))))
	       (unify-and-subst
		;; TYPE: (-> (type-expr (env uniq-sym type-expr))
		;;           (pair type-expr
		;;                 (env uniq-sym type-expr)))
		(lambda (ut ne)
		  (let ((bindings (tc:unify ut abst-type ne '())))
		    (if (equal? *tc:unification-failure* bindings)
			;; If unification failed then return type-expr
			;; and environment without changes
			(cons ut ne)
			;; If unification succeeded then replace the
			;; type-expr by the representation of the abstract
			;; type with bindings substituted
			(cons
			 (tc:subst-bindings (something->value bindings)
					    rep-type)
			 (something->value bindings)))))))
	(cond
	 ((tc:variable-type-expr? up-type) (cons up-type env))
	 ((tc:basic-type-expr? up-type)
	  (cons (if (and (tc:basic-type-expr? abst-type)
			 (tc:basic-type-expr-eq? up-type abst-type))
		    rep-type
		    up-type)
		env))
	 ((tc:function-type-expr? up-type)
	  (let* ((res-args (down-type-list
			    (tc:function-type-expr->arg-types up-type) env))
		 (res-result (tc:deep-down-type
			      (tc:function-type-expr->result-type up-type)
			      uniq-rep
			      (cdr res-args))))
	    (unify-and-subst (tc:make-function-type-expr (car res-args)
							 (car res-result))
			     (cdr res-result))))
	 ((tc:and-type-expr? up-type)
	  (let ((res-conj (down-type-list
			   (tc:and-type-expr->conjoined-types up-type) env)))
	    (unify-and-subst (tc:make-and-type-expr (car res-conj))
			     (cdr res-conj))))
	 ((tc:applied-type-expr? up-type)
	  (let* ((res-rator (tc:deep-down-type
			     (tc:applied-type-expr->operator-type up-type)
			     uniq-rep
			     env))
		 (res-rands (down-type-list
			     (tc:applied-type-expr->operand-types up-type)
			     (cdr res-rator))))
	    (unify-and-subst (tc:make-applied-type-expr
			      (car res-rator)
			      (car res-rands))
			     (cdr res-rands))))
	 (else (error "tc:deep-down-type: invalid type expr: " up-type)))))))


